package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.SigningRecordFormDTO;
import digital.laboratory.platform.reagent.entity.SignedBatchList;
import digital.laboratory.platform.reagent.entity.SigningRecordForm;
import digital.laboratory.platform.reagent.service.SigningRecordFormService;
import digital.laboratory.platform.reagent.vo.SigningRecordFormVO;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;
import java.util.List;

/**
 * 签收记录表
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收记录表 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/signing_record_form")
@Api(value = "signing_record_form", tags = "签收记录表管理")
public class SigningRecordFormController {

    private final SigningRecordFormService signingRecordFormService;

    /**
     * 通过id查询签收记录表
     *
     * @param signingRecordFormId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{signingRecordFormId}")
//	@PreAuthorize("@pms.hasPermission('reagent_signing_record_form_get')" )
    public R<SigningRecordForm> getById(@PathVariable("signingRecordFormId") String signingRecordFormId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SigningRecordFormVO signingRecordFormVO = signingRecordFormService.getSigningRecordFormVO(signingRecordFormId);

        return R.ok(signingRecordFormVO);
    }

    /**
     * 分页查询
     *
     * @param page              分页对象
     * @param signingRecordForm 签收记录表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
//    @PreAuthorize("@pms.hasPermission('reagent_signing_record_form_get')")
    public R<IPage<SigningRecordForm>> getSigningRecordFormVOPage(Page<SigningRecordForm> page, SigningRecordForm signingRecordForm, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<SigningRecordForm> signingRecordFormPage = signingRecordFormService.page(page, Wrappers.<SigningRecordForm>query()
                .eq("create_by", dlpUser.getId())
                .orderByDesc("create_time"));

        return R.ok(signingRecordFormPage);

    }


    /**
     * 新增签收记录表
     *
     * @param signingRecordFormDTOList 签收记录表
     * @return R
     */
    @ApiOperation(value = "新增签收记录表", notes = "新增签收记录表")
    @SysLog("新增签收记录表")
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('reagent_signing_record_form_add')")
    public R<String> postAddObject(@RequestBody List<SigningRecordFormDTO> signingRecordFormDTOList, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        Boolean aBoolean = signingRecordFormService.addFormById(signingRecordFormDTOList, dlpUser);


        if (aBoolean) {
            return R.ok("保存成功");
        } else {
            return R.failed("保存失败");
        }
    }

    /**
     * 修改签收记录表
     *
     * @param signingRecordFormDTO 签收记录表
     * @return R
     */
    @ApiOperation(value = "修改签收记录表", notes = "修改签收记录表")
    @SysLog("修改签收记录表")
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('reagent_signing_record_form_edit')")
    public R<SignedBatchList> putUpdateById(@RequestBody SigningRecordFormDTO signingRecordFormDTO, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SignedBatchList signedBatchList = signingRecordFormService.editFormById(signingRecordFormDTO);

        if (signedBatchList != null) {
            return R.ok(signedBatchList, "修改成功");
        } else {
            return R.failed(signedBatchList, "修改失败");
        }
    }

    /**
     * 通过id删除签收记录表
     *
     * @param signingRecordFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除签收记录表", notes = "通过id删除签收记录表")
    @SysLog("通过id删除签收记录表")
    @DeleteMapping("/{signingRecordFormId}")
    @PreAuthorize("@pms.hasPermission('reagent_signing_record_form_del')")
    public R<SigningRecordForm> deleteById(@PathVariable String signingRecordFormId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SigningRecordForm oldSigningRecordForm = signingRecordFormService.getById(signingRecordFormId);

        if (signingRecordFormService.removeById(signingRecordFormId)) {
            return R.ok(oldSigningRecordForm, "对象删除成功");
        } else {
            return R.failed(oldSigningRecordForm, "对象删除失败");
        }

    }

}
